<?php
declare(strict_types = 1);

/**
 * GroupBallotCreateMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending, and receiving Threema messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Messages\ThreemaGroupMessage;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\DeliveryReceiptType;
use Threema\MsgApi\Types\FileRenderingType;
use Threema\MsgApi\Types\Voting\BallotChoice;

/**
 * GroupBallotCreateMessage Class Doc Comment
 *
 * @category Class
 * @description A group poll create message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class GroupBallotCreateMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x52;

	/**
	 * Constructor
	 *
	 */
	public function __construct(
		private readonly GroupId $groupId,
		private readonly BallotId $ballotId,
		private readonly string $description,
		private readonly \Threema\MsgApi\Types\Voting\State $state,
		private readonly \Threema\MsgApi\Types\Voting\VotingMode $votingMode,
		private readonly \Threema\MsgApi\Types\Voting\ResultsDisclosureType $resultsDisclosureType,
		private readonly \Threema\MsgApi\Types\Voting\DisplayMode $displayMode,
		private readonly array $choices,
		private readonly ?array $participants = null) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([


					'd' => $this->getDescription(),
					's' => $this->getState()->value,
					'a' => $this->getVotingMode()->value,
					't' => $this->getResultsDisclosureType()->value,
					'u' => $this->getDisplayMode()->value,
			'c' => array_values(
				array_map(
					static fn(BallotChoice $choices) => array_filter($choices->__toArray(), static fn($v) => Util::array_filter_callback($v)),
					$this->getChoices()
				)
			)
		,
					'o' => 0,
					'p' => $this->getParticipants()

		], static fn($v) => Util::array_filter_callback($v));
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 * @return string|array
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupBallotCreateMessage.
	 *
	 * @throws \Exception
	 */
	public static function fromString(string $data, int $realDataLength, \Closure $piece): GroupBallotCreateMessage
	{
		Serializers\GroupBallotCreateMessageSerializer::validate($data, $realDataLength);
		return Serializers\GroupBallotCreateMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->ballotId) {
			$invalidProperties[] = "'ballotId' can't be null";
		}
		if (null === $this->description) {
			$invalidProperties[] = "'description' can't be null";
		}
		if (null === $this->state) {
			$invalidProperties[] = "'state' can't be null";
		}
		if (null === $this->votingMode) {
			$invalidProperties[] = "'votingMode' can't be null";
		}
		if (null === $this->resultsDisclosureType) {
			$invalidProperties[] = "'resultsDisclosureType' can't be null";
		}
		if (null === $this->displayMode) {
			$invalidProperties[] = "'displayMode' can't be null";
		}
		if (null === $this->choices) {
			$invalidProperties[] = "'choices' can't be null";
		}
		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}


	/**
	 * Gets groupId
	 *
	 * @return GroupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets ballotId
	 *
	 * @return BallotId
	 */
	public function getBallotId(): BallotId
	{
		return $this->ballotId;
	}

	/**
	 * Gets description
	 *
	 * @return string
	 */
	public function getDescription(): string
	{
		return $this->description;
	}

	/**
	 * Gets state
	 *
	 * @return \Threema\MsgApi\Types\Voting\State
	 */
	public function getState(): \Threema\MsgApi\Types\Voting\State
	{
		return $this->state;
	}

	/**
	 * Gets votingMode
	 *
	 * @return \Threema\MsgApi\Types\Voting\VotingMode
	 */
	public function getVotingMode(): \Threema\MsgApi\Types\Voting\VotingMode
	{
		return $this->votingMode;
	}

	/**
	 * Gets resultsDisclosureType
	 *
	 * @return \Threema\MsgApi\Types\Voting\ResultsDisclosureType
	 */
	public function getResultsDisclosureType(): \Threema\MsgApi\Types\Voting\ResultsDisclosureType
	{
		return $this->resultsDisclosureType;
	}

	/**
	 * Gets displayMode
	 *
	 * @return \Threema\MsgApi\Types\Voting\DisplayMode
	 */
	public function getDisplayMode(): \Threema\MsgApi\Types\Voting\DisplayMode
	{
		return $this->displayMode;
	}

	/**
	 * Gets choices
	 *
	 * @return array
	 */
	public function getChoices(): array
	{
		return $this->choices;
	}

	/**
	 * Gets participants
	 *
	 * @return array|null
	 */
	public function getParticipants(): ?array
	{
		return $this->participants;
	}

	/**
	 * Gets the string presentation of the object
	 *
	 * @return string
	 */
	public function __toString(): string
	{
		return 'GroupBallotCreateMessage{' . 'groupId=' . $this->groupId . ', '
		. 'ballotId=' . $this->ballotId . ', '
		. 'description=' . $this->description . ', '
		. 'state=' . $this->state->name . ', '
		. 'votingMode=' . $this->votingMode->name . ', '
		. 'resultsDisclosureType=' . $this->resultsDisclosureType->name . ', '
		. 'displayMode=' . $this->displayMode->name . ', '
		. 'choices=' . print_r($this->choices, true) . ', '
		. 'order=' . $this->order . ', '
		. 'participants=' . print_r($this->participants, true) . '}';
	}
}
