<?php
declare(strict_types = 1);

/**
 * GroupRenameMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending, and receiving Threema messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Messages\ThreemaGroupMessage;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\DeliveryReceiptType;
use Threema\MsgApi\Types\FileRenderingType;
use Threema\MsgApi\Types\Voting\BallotChoice;

/**
 * GroupRenameMessage Class Doc Comment
 *
 * @category Class
 * @description A group rename message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class GroupRenameMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x4b;

	/**
	 * Constructor
	 *
	 */
	public function __construct(
		private readonly GroupId $groupId,
		private readonly string $groupName) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([



		], static fn($v) => Util::array_filter_callback($v));
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 * @return string|array
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupRenameMessage.
	 *
	 * @throws \Exception
	 */
	public static function fromString(string $data, int $realDataLength, \Closure $piece, ?string $creator = null): GroupRenameMessage
	{
		Serializers\GroupRenameMessageSerializer::validate($data, $realDataLength);
		return Serializers\GroupRenameMessageSerializer::deserialize($data, $piece, $creator);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->groupName) {
			$invalidProperties[] = "'groupName' can't be null";
		}
		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}


	/**
	 * Gets groupId
	 *
	 * @return GroupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets groupName
	 *
	 * @return string
	 */
	public function getGroupName(): string
	{
		return $this->groupName;
	}

	/**
	 * Gets the string presentation of the object
	 *
	 * @return string
	 */
	public function __toString(): string
	{
		return 'GroupRenameMessage{' . 'groupId=' . $this->groupId . ', '
		. 'groupName=' . $this->groupName . '}';
	}
}
