<?php
declare(strict_types = 1);

/**
 * LocationMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending, and receiving Threema messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Messages\ThreemaMessage;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\DeliveryReceiptType;
use Threema\MsgApi\Types\FileRenderingType;
use Threema\MsgApi\Types\Voting\BallotChoice;

/**
 * LocationMessage Class Doc Comment
 *
 * @category Class
 * @description A Location Message
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class LocationMessage extends ThreemaMessage
{
	public const TYPE_CODE = 0x10;

	/**
	 * Constructor
	 *
	 */
	public function __construct(
		private readonly string $latitude,
		private readonly string $longitude,
		private readonly ?string $address = null,
		private readonly ?float $accuracy = null,
		private readonly ?string $poiName = null) {
	}

	public function __toArray(): array
	{
		return array_filter([






		], static fn($v) => Util::array_filter_callback($v));
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 * @return string|array
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized LocationMessage.
	 *
	 * @throws \Exception
	 */
	public static function fromString(string $data, int $realDataLength, \Closure $piece): LocationMessage
	{
		Serializers\LocationMessageSerializer::validate($data, $realDataLength);
		return Serializers\LocationMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->latitude) {
			$invalidProperties[] = "'latitude' can't be null";
		}
		if (null === $this->longitude) {
			$invalidProperties[] = "'longitude' can't be null";
		}
		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}


	/**
	 * Gets latitude
	 *
	 * @return string
	 */
	public function getLatitude(): string
	{
		return $this->latitude;
	}

	/**
	 * Gets longitude
	 *
	 * @return string
	 */
	public function getLongitude(): string
	{
		return $this->longitude;
	}

	/**
	 * Gets accuracy
	 *
	 * @return float|null
	 */
	public function getAccuracy(): ?float
	{
		return $this->accuracy;
	}

	/**
	 * Gets poiName
	 *
	 * @return string|null
	 */
	public function getPoiName(): ?string
	{
		return $this->poiName;
	}

	/**
	 * Gets address
	 *
	 * @return string|null
	 */
	public function getPoiAddress(): ?string
	{
		return $this->address;
	}

	/**
	 * Gets the string presentation of the object
	 *
	 * @return string
	 */
	public function __toString(): string
	{
		return 'LocationMessage{' . 'latitude=' . $this->latitude . ', '
		. 'longitude=' . $this->longitude . ', '
		. 'accuracy=' . $this->accuracy . ', '
		. 'poiName=' . $this->poiName . ', '
		. 'address=' . $this->address . '}';
	}
}
