<?php
declare(strict_types = 1);

/**
 * TextMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending, and receiving Threema messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Messages\ThreemaMessage;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\DeliveryReceiptType;
use Threema\MsgApi\Types\FileRenderingType;
use Threema\MsgApi\Types\Voting\BallotChoice;

/**
 * TextMessage Class Doc Comment
 *
 * @category Class
 * @description A text message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class TextMessage extends ThreemaMessage
{
	public const TYPE_CODE = 0x01;
	private readonly string $text;

	/**
	 * Constructor
	 *
	 */
	public function __construct(
		string $text,
		?string $quotedMessageId = null
	) {
		if (null !== $quotedMessageId) {
			$this->text = Util::formatAsQuote($quotedMessageId, $text);
		} else {
			$this->text = $text;
		}
	}

	public function __toArray(): array
	{
		return array_filter([


		], static fn($v) => Util::array_filter_callback($v));
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 * @return string|array
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized TextMessage.
	 *
	 * @throws \Exception
	 */
	public static function fromString(string $data, int $realDataLength, \Closure $piece): TextMessage
	{
		Serializers\TextMessageSerializer::validate($data, $realDataLength);
		return Serializers\TextMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->text) {
			$invalidProperties[] = "'text' can't be null";
		}
		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}


	/**
	 * Gets text
	 *
	 * @return string
	 */
	public function getText(): string
	{
		return $this->text;
	}

	/**
	 * Whether the message is a quote
	 */
	public function isQuote(): bool
	{
		$matches = preg_match(Util::QUOTE_PATTERN, $this->text);
		return !(false === $matches || $matches === 0);
	}

	/**
	 * Gets quoted message id if the message is a quote
	 */
	public function getQuotedMessageId(): ?string
	{
		if (false === $this->isQuote()) {
			return null;
		}

		return Util::extractQuote($this->text, \Threema\MsgApi\Types\QuotePart::QUOTED_MESSAGE_ID);
	}

	/**
	 * Gets quoted text if the message is a quote
	 */
	public function getQuoteText(): ?string
	{
		if (false === $this->isQuote()) {
			return null;
		}

		return Util::extractQuote($this->text, \Threema\MsgApi\Types\QuotePart::QUOTE_TEXT);
	}

	/**
	 * Gets the string presentation of the object
	 *
	 * @return string
	 */
	public function __toString(): string
	{
		return 'TextMessage{' . 'text=' . $this->text . '}';
	}
}
