<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Tools;

use Threema\MsgApi\Exceptions\InvalidArgumentException;
use Threema\MsgApi\Types\QuotePart;

class Util
{
	public const QUOTE_PATTERN = '/^> quote #([[:xdigit:]]{16})(?:\r?\n){2}(.+)$/s';

	public static function array_filter_callback(mixed $v): bool
	{
		return !(is_null($v) or '' === $v);
	}

	public static function extractQuote(string $text, QuotePart $part): ?string
	{
		$matches = [];
		preg_match(self::QUOTE_PATTERN, $text, $matches, PREG_UNMATCHED_AS_NULL);

		if (array_key_exists($part->value, $matches) && !empty($matches[$part->value])) {
			return $matches[$part->value];
		}

		return null;
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public static function formatAsQuote(string $quoteMessageId, string $text): string
	{
		if (1 === preg_match('/^[[:xdigit:]]{16}$/', $quoteMessageId)) {
			return '> quote #' . $quoteMessageId . PHP_EOL . PHP_EOL . $text;
		}
		throw new InvalidArgumentException('invalid message id format');
	}
}
